IND0      equ     	0h
RTCC      equ     	1h
PCL       equ     	2h
STATUS    equ     	3h              ; F3 Reg is STATUS Reg.
FSR       equ     	4h
;
PORT_A    equ     	5h
PORT_B    equ     	6h              ; I/O Port Assignments
;
EEDATA    equ     	08h
EEADR     equ     	09h
PCLATH    equ     	0Ah             ;16C71 Special-purpose registers
INTCON    equ     	0Bh             ;16C71 Special-purpose registers
TRISA     equ     	05h             ;16C71 Special-purpose registers
TRISB     equ     	06h             ;16C71 Special-purpose registers
EECON1    equ     	08h             ;16C84
EECON2    equ     	09h
EEROM     EQU       PORT_A
LED       EQU       PORT_B

C	EQU	0
Z	EQU	2
M	EQU	0
L	EQU	1
N	EQU	2
MS	EQU	6
LS	EQU	7
DO        EQU       0
DI        EQU       1
SK        EQU       2
SDA       EQU       1
SCL       EQU       2
SW_A      EQU       3
SW_B      EQU       4
RD	EQU	0
WR	EQU	1
WREN	EQU	2
WRERR	EQU	3
EEIF	EQU	4
MASTER  	EQU     	80H
CHP_CNT   EQU       10H
CMD_CNT   EQU       11H
DSP_CNT   EQU       12H
BYT_CNT   EQU       14H
CNT       EQU       15H
CNT_HI    EQU       16H
BUFFER    EQU       17H
EE_ADR    EQU       19H
SLAVE   	EQU     	1AH
TEMP    	EQU       1BH
DATA_H    EQU       1CH
DATA_L    EQU       1DH
DATA_VH   EQU       1EH
DATA_VL   EQU       1FH
PWR_SAV1	EQU	21H
PWR_SAV2	EQU	22H
CNFG_CNT	EQU	23H
LOCK_BYT	EQU	24H
EE_SEL	EQU	25H
CMDFLAG   EQU       26H

;***********************************************************
SET_SDA_OUT MACRO
	BSF       STATUS,5
	BCF       TRISA,SDA
	BCF       STATUS,5
	ENDM

SET_SDA_IN  MACRO
	BSF       STATUS,5
	BSF       TRISA,SDA
	BCF       STATUS,5
	ENDM

START_BIT MACRO
	BSF       EEROM,SDA       ;START BIT
	BSF       EEROM,SCL
	NOP
	BCF       EEROM,SDA
	NOP
	NOP
	BCF       EEROM,SCL
	ENDM

STOP_BIT  MACRO
	BCF       EEROM,SDA
	NOP
	BSF       EEROM,SCL
	NOP
	NOP
	BSF       EEROM,SDA
	ENDM

WAIT_ACK  MARCO
          BSF       EEROM,SCL
	MOVLW     B'01111000'     ;NO, PRINT ERROR
	MOVWF	BUFFER          ;AND RESTART
	BTFSC     EEROM,SDA
	GOTO      WERM
	BCF       EEROM,SCL
          ENDM

SND_DRV_ADR   MARCO
          BSF      EEROM,SDA            ;send 1 0 1 0
          CALL     CLOCK
          BCF      EEROM,SDA
          CALL     CLOCK
          BSF      EEROM,SDA
          CALL     CLOCK
          BCF      EEROM,SDA
          CALL     CLOCK

          BSF      EEROM,SDA            ;assume device address = 110
          CALL     CLOCK
          CALL     CLOCK
          BCF      EEROM,SDA
          CALL     CLOCK
          ENDM

SND_C01_ADR  MACRO
	LOCAL     C01ADR_LP
	MOVF    	EE_ADR,0         ;KEEPS ADDRESS
	MOVWF   	TEMP    	       ;SEND ADDRESS FIRST
	MOVLW   	8
	MOVWF   	BYT_CNT
	RLF   	TEMP,1           ;SHIFT MSB TO CARRY
C01ADR_LP BCF	EEROM,SDA
	RLF   	TEMP,1           ;SHIFT MSB TO CARRY
	BTFSC     STATUS,0         ;TEST CARRY BIT, SET IF BIT = 1
	BSF	EEROM,SDA
	CALL	CLOCK
	DECFSZ	BYT_CNT
	GOTO	C01ADR_LP
	ENDM
;***********************************************************
	ORG       0               	;RESET VECTOR 0000
	GOTO    	COLD            	;GOTO POWER ON MEUN
	ORG       4               	;INTERRUPT VECTOR 0004
DISPLAY                             	;THIS ROUTINE WILL AUTO RUN
	BCF       INTCON,2        	;EVERY 65MS    // XTAL 4MHZ
	DECFSZ	PWR_SAV1
	GOTO	CONFIG
	DECFSZ    PWR_SAV2
	GOTO	CONFIG              ;IF PWR_SAV REACH
	MOVLW	0FFH
	MOVWF	LED                 ;TURNS OFF LED &
	SLEEP                         ;GOODBYE
CONFIG	BTFSC	EEROM,SW_A
	CLRF	CNFG_CNT            ;IF CONTINUEOUSLY PRESSING
	BTFSS	EEROM,SW_A          ;SW_A FOR 10 SECOND
	DECFSZ	CNFG_CNT            ;SET CMDFLAG LOCK BIT
	GOTO	DSP_MAIN            ;AND RESTART
	CLRF      CMDFLAG
	BSF       CMDFLAG,L
	CLRF	CMD_CNT
	CLRF	CHP_CNT
	MOVLW	3AH
	MOVWF	BUFFER
DSP_MAIN
	DECFSZ    DSP_CNT
	GOTO      DSP_END
	MOVF      BUFFER,0        ;
	XORLW     080H            ;display_count = 8;
	MOVWF     BUFFER
	COMF	BUFFER,0
	MOVWF     LED
	MOVLW     10H
	MOVWF     DSP_CNT
DSP_END   CLRF      RTCC            ;RESET RTCC
	RETFIE
;**************************************************************
COLD
	CLRF     	BUFFER
	MOVLW	1
          CALL	READ_EE
	MOVWF	LOCK_BYT
	MOVLW	2
          CALL	READ_EE
	MOVWF	EE_SEL
WERM
	BSF       STATUS,5        ;SET PAGE 1
	MOVLW     B'00011001'
	MOVWF     TRISA           ;PORT A ALL INPUT
	CLRF      TRISB	      ;PORT B ALL OUTPUT
	MOVLW     B'00000101'     ;PRESCALAR SETTING 1:128
	OPTION    	      ;RTCC INTERNAL CKS
	BCF       STATUS,5        ;BACK TO PAGE 0
	MOVLW     B'10100000'     ;ENABLE RTCC INTERRUPT
	MOVWF     INTCON
	CLRF      RTCC
	CLRF      CHP_CNT
	CLRF      CMD_CNT
	CLRF	PWR_SAV1
	MOVLW	20
	MOVWF	PWR_SAV2
	MOVLW     10H
	MOVWF     DSP_CNT
	MOVLW     B'00000001'
	MOVWF     CMDFLAG
	MOVLW     7
	MOVWF     BYT_CNT
;**************************************************************
;         C M D F L A G	REPRESENTITION
;    BIT  0 = M  SCAN MAIN JOB
;	1 = L  SCAN LOCK JOB
;	2 = N  SCAN NUMBER COUNT
;         3 = SW_A
;	4 = SW_B
;	6 = MS MAIN JOB INPUT OKAY
;	7 = LS LOCK/UNLOCK CHANGE INPUT OKAY

MAIN      BTFSS     CMDFLAG,M       ;This is a loop job scan
	GOTO      LOCK_CGE        ;first bit set is main job
	CALL      CHK_KEY         ;keyboard check
MAINA     BTFSS     CMDFLAG,SW_A
	GOTO      MAINB
	BTFSS	CMDFLAG,M
	GOTO	LOCK_CGE	      ;JOB MAST CHANGE BY INTERRUPT
	INCF      CHP_CNT
	MOVF      CHP_CNT,0
	SUBLW     3               ;MAIN MENU HAS 3 SELECTION ONLY
	BTFSC     STATUS,C
	GOTO      CNVTMENU
	MOVLW     1
	MOVWF     CHP_CNT
CNVTMENU
	MOVF	CHP_CNT,0
	CALL      MAIN_MENU
	MOVWF     BUFFER
MAINB     BTFSS     CMDFLAG,SW_B
	GOTO      LOCK_CGE
	MOVF      CHP_CNT,0
	MOVWF     CMD_CNT
	SUBLW     3
	BTFSS     STATUS,Z
	GOTO      SKIP_3
	MOVLW     3
	MOVWF     BYT_CNT
SKIP_3    CLRF      CMDFLAG             ;TURN OFF MAIN JOB
	BSF       CMDFLAG,MS          ;MAIN OKAY
	BSF       CMDFLAG,N           ;TURN ON NUMBER CHK
	GOTO 	RESET1              ;SET DEFAULT TO 1
LOCK_CGE  BTFSS     CMDFLAG,L
	GOTO      ALP_CHK
	CALL      CHK_KEY
LOCKA     BTFSS     CMDFLAG,SW_A
	GOTO      LOCKB
	INCF      CHP_CNT
	MOVF      CHP_CNT,0
	SUBLW     2                ;LOCK MENU HAS 2 SELECTION ONLY
	BTFSC     STATUS,C
	GOTO      CNVTLOCK
	MOVLW     1
	MOVWF     CHP_CNT
CNVTLOCK  MOVF	CHP_CNT,0
	CALL      LOCK_MENU
	MOVWF     BUFFER
LOCKB     BTFSS     CMDFLAG,SW_B
	GOTO      ALP_CHK
	MOVF      CHP_CNT,0
	MOVWF     CMD_CNT
	CLRF      CMDFLAG          ;TURN OFF MAIN JOB
	BSF	CMDFLAG,LS
	BSF       CMDFLAG,N        ;TURN ON NUMBER CHK
	GOTO	RESET1
ALP_CHK   BTFSS     CMDFLAG,N
	GOTO      CHECK_END
	CALL      CHK_KEY
ALPA      BTFSS     CMDFLAG,SW_A
	GOTO      ALPB
	INCF      CHP_CNT
	MOVF      CHP_CNT,0
	SUBWF     BYT_CNT,0         ;LOCK MENU HAS 2 SELECTION ONLY
	BTFSC     STATUS,C
	GOTO      CNVTALP
RESET1	MOVLW     1
	MOVWF     CHP_CNT
CNVTALP   MOVF	CHP_CNT,0
	CALL      ALP_MENU
	MOVWF     BUFFER
	BCF	CMDFLAG,SW_B
ALPB      BTFSS     CMDFLAG,SW_B
CHECK_END GOTO      MAIN
	GOTO      JOB_FND

ALP_MENU  ADDWF     PCL,1           	;PCL = FW
	retlw     03FH		;0C0H ;0
	retlw     006H		;0F9H ;1
	retlw     05BH		;0A4H ;2
	retlw     04FH		;0B0H ;3
	retlw     066H		;099H ;4
	retlw     06DH		;092H ;5
	retlw     07DH		;082H ;6
	retlw     007H		;0F8H ;7
	retlw     07FH		;080H ;8
	retlw     06FH		;090H ;9

MAIN_MENU ADDWF     PCL,1
	retlw     000H
	retlw     073H		;08CH ;P
	retlw     039H		;0C6H ;C
	retlw     079H		;E

LOCK_MENU ADDWF     PCL,1
	retlw     000H
	retlw     038H		;L
	retlw     03EH		;U
	RETLW     05CH                ;o

AG_MENU   ADDWF     PCL,1           	;PCL = FW
	retlw     03FH		;0C0H ;0
	retlw     077H		;A
	retlw     07CH		;B
	retlw     039H		;C
	retlw     05EH		;D
	retlw     079H		;E
	retlw     071H		;F
	retlw     07DH		;G

CHK_KEY   BCF      	CMDFLAG,SW_A
	BTFSC     PORT_A,SW_A
	GOTO      CHK_B
	CALL      DELAY
WAIT_A    BTFSS     PORT_A,SW_A
	GOTO      WAIT_A
	BSF       CMDFLAG,SW_A
	MOVLW     20H
	MOVWF     PWR_SAV2            ;RESET POWER SAVE FLAG
CHK_B     BTFSC     PORT_A,SW_B
	GOTO      NO_KEY
	CALL      DELAY
WAIT_B    BTFSS     PORT_A,SW_B
	GOTO      WAIT_B
	MOVF      CHP_CNT,1
	BTFSS     STATUS,Z
	BSF       CMDFLAG,SW_B
NO_KEY    RETURN

JOB_FND   BCF       INTCON,7        ;DISABLE ALL INTERRUPT
	CLRF	LED
;CONVERT CHIP_COUNT TO MATCH HARDWARE CONNECTION
	MOVF	CHP_CNT,0
	MOVWF	TEMP
	CLRF    	SLAVE
	BSF     	STATUS,C
CHK_CHP 	RLF     	SLAVE,1
	BCF     	STATUS,C
	DECFSZ    TEMP         ;DECODE CHIP SELECT
	GOTO    	CHK_CHP
	BTFSS	CMDFLAG,MS
	GOTO	CFG_FND

	MOVF	EE_SEL,0
	MOVWF	TEMP
	DECFSZ    TEMP,1
	GOTO	NEXT_E1
	GOTO	C66_P
NEXT_E1	DECFSZ	TEMP,1
	GOTO	NEXT_E2
	GOTO	C46_P
NEXT_E2	DECFSZ	TEMP,1
	GOTO	EE_CHGE
	GOTO	C01_P

EE_CHGE
	BCF	INTCON,7
	MOVF	CHP_CNT,0
	MOVWF	EEDATA
	MOVLW	2
          CALL	WRITE_EE
	MOVLW	2
          CALL	READ_EE
	MOVWF	EE_SEL
	CALL	AG_MENU
	MOVWF	BUFFER
	GOTO	WERM

CFG_FND	DECFSZ    CMD_CNT
	GOTO      CHG_2
LOCK      MOVF	SLAVE,0
	IORWF	LOCK_BYT,0
	MOVWF	EEDATA
	MOVLW	1
	CALL      WRITE_EE
	MOVLW     1
	CALL      READ_EE
	MOVWF	LOCK_BYT
	MOVWF	BUFFER
	GOTO 	WERM
CHG_2     DECFSZ    CMD_CNT
	GOTO      COLD
UNLOCK    COMF	SLAVE,0
	ANDWF	LOCK_BYT,0
	MOVWF	EEDATA
	MOVLW	1
	CALL      WRITE_EE
	MOVLW	1
	CALL      READ_EE
	MOVWF	LOCK_BYT
	MOVWF	BUFFER
	GOTO	WERM

CHK_LOCK
	MOVF	LOCK_BYT,0
	MOVWF	TEMP
	MOVF	SLAVE,0
	ANDWF	TEMP,1
	BTFSC     STATUS,2
	RETURN		;NO LOCK RETURN
LOCKED    MOVLW	038H	;PRINT L MESSAGE
	MOVWF	BUFFER
	GOTO	WERM

;DECODE COMMAND NUMBER
C66_P	DECFSZ    CMD_CNT         ;DECODE CHIP SELECT
	GOTO      C66_L
	GOTO      PROGC66         ;FIRST JOB
C66_L     DECFSZ    CMD_CNT
	GOTO      C66_V
	GOTO      LOADC66         ;SECOND JOB
C66_V     DECFSZ    CMD_CNT
	GOTO      COLD
	GOTO	EE_CHGE

C46_P	DECFSZ    CMD_CNT         ;DECODE CHIP SELECT
	GOTO      C46_L
	GOTO      PROGC46         ;FIRST JOB
C46_L     DECFSZ    CMD_CNT
	GOTO      C46_V
	GOTO      LOADC46         ;SECOND JOB
C46_V     DECFSZ    CMD_CNT
	GOTO      COLD
	GOTO	EE_CHGE

C01_P	DECFSZ    CMD_CNT         ;DECODE CHIP SELECT
	GOTO      C01_L
	GOTO      PROGC01         ;FIRST JOB
C01_L     DECFSZ    CMD_CNT
	GOTO      C01_V
	GOTO      LOADC01         ;SECOND JOB
C01_V     DECFSZ    CMD_CNT
	GOTO      COLD
	GOTO	EE_CHGE

VERIC66
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0         ;COPY START FROM 0 BYTE
	MOVWF     EE_ADR
C66V_LP    MOVLW     MASTER    ;READ MASTER
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVF      DATA_H,0  ;SAVE IN DATA_VH
	MOVWF     DATA_VH
	MOVF      DATA_L,0  ;SAVE IN DATA_VL
	MOVWF     DATA_VL
	MOVF      SLAVE,0   ;READ SLAVE
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVF      DATA_VH,0
	SUBWF     DATA_H
	BTFSS     STATUS,2
	GOTO      ERROR
	MOVF      DATA_VL,0
	SUBWF     DATA_L
	BTFSS     STATUS,2
	GOTO      ERROR
	INCFSZ    EE_ADR    ;NEXT BYTE UNTIL 512 BYTE
	GOTO      C66V_LP
	MOVLW	B'01101110'         ;YES , OKAY
	MOVWF	BUFFER
	GOTO      DISABLE
ERROR     MOVLW     B'01111001'	;E , ERROR
	MOVWF	BUFFER
DISABLE	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWDS
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWDS
	GOTO      WERM

PROGC66
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	CALL	ERAL
          CALL      DELAY
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0
	MOVWF     EE_ADR
C66P_LP   MOVF      SLAVE,0
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVLW     MASTER
	MOVWF     CHP_CNT
	CALL      WRABC66
	INCFSZ    EE_ADR
	GOTO      C66P_LP
	GOTO	VERIC66

LOADC66
	CALL      CHK_LOCK
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0         ;COPY START FROM 0 BYTE
	MOVWF     EE_ADR
C66L_LP   MOVLW     MASTER    ;READ MASTER
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVF      SLAVE,0   ;WRITE SLAVE
	MOVWF     CHP_CNT
	CALL      WRABC66
	INCFSZ    EE_ADR    ;NEXT BYTE UNTIL 512 BYTE
	GOTO      C66L_LP
	GOTO      VERIC66

CLOCK
	BSF       EEROM,SK
	NOP
	BCF       EEROM,SK
	RETURN

EWEN
	MOVF	CHP_CNT,0
	MOVWF	LED
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CLRF	LED
	RETURN

ERAL
	MOVF	CHP_CNT,0
	MOVWF	LED
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	BSF	EEROM,DI
	CALL 	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CLRF	LED
	RETURN

EWDS
	MOVF	CHP_CNT,0
	MOVWF	LED
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	CALL 	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CALL	CLOCK
	CLRF	LED
	RETURN

WRABC66
	MOVF    	CHP_CNT,0
	MOVWF     LED             ;CHIP SELECT
          CALL	WRITE_CMD
	MOVF    	EE_ADR,0
	MOVWF   	TEMP    	;WRITE ADDRESS TO DI
	CALL	WRITE_DI
	CALL	WR2BYT
	CLRF      LED
	RETURN

RDABC66
	MOVF    	CHP_CNT,0	;ENABLE CHIP SELECTED
	MOVWF	LED
	CALL      READ_CMD
	MOVF    	EE_ADR,0  ;KEEPS ADDRESS
	MOVWF   	TEMP    	;SEND ADDRESS FIRST
	CALL	WRITE_DI
	CALL	RD2BYT
          CLRF      LED
	RETURN

RD2BYT	CALL	READ_DO  	;THEN READ A BYTE
	MOVF    	TEMP,0    ;DATA COMES BACK IN TEMP
	MOVWF   	DATA_H    ;MOST SIGNIFIENT BYTE FIRST
	CALL	READ_DO  	;GETS LOWER BYTE
	MOVF    	TEMP,0
	MOVWF   	DATA_L
	RETURN

WR2BYT	MOVF    	DATA_H,0
	MOVWF   	TEMP      ;WRITE HIGHER BYTE
	CALL	WRITE_DI
	MOVF    	DATA_L,0
	MOVWF   	TEMP      ;WRITE LOWER BYTE
	CALL	WRITE_DI
RDYCHK
	MOVLW     40H       ;TRIES 20H X 256 TIMES
	MOVWF     CNT_HI    ;20H
	CLRF      CNT       ;LSB OF ERROR TRIES
	CLRF    	LED	;AFTER DATA ARE WRITEN TO DI
	MOVF    	CHP_CNT,0	;CHIP SEL NEEDS TO DROP 1 CLOCK TIME
	MOVWF	LED       ;AT LEAST. THEN WAIT FOR DO
NOTRDY    BTFSC     EEROM,DO
	RETURN
	DECFSZ    CNT
	GOTO      NOTRDY
	DECFSZ    CNT_HI
	GOTO      NOTRDY
	MOVLW     B'01111001'	;YES, OKAY
	MOVWF	BUFFER
	GOTO      WERM

READ_DO   MOVLW   	8
	MOVWF   	BYT_CNT
MOV_D1    RLF	TEMP
	BCF   	TEMP,0
	CALL	CLOCK
	BTFSC	EEROM,DO
	BSF	TEMP,0
	DECFSZ    BYT_CNT
	GOTO      MOV_D1
	RETURN
WRITE_DI
	MOVLW   	8
	MOVWF   	BYT_CNT
MOV_T1    BCF	EEROM,DI
	RLF   	TEMP,1    ;SHIFT MSB TO CARRY
	BTFSC     STATUS,0  ;TEST CARRY BIT, SET IF BIT = 1
	BSF	EEROM,DI
	CALL	CLOCK
	DECFSZ	BYT_CNT
	GOTO	MOV_T1
	BCF	EEROM,DI
	RETURN
PROGC46
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	CALL	ERAL
          CALL      DELAY
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0XC0
	MOVWF     EE_ADR
C46P_LP   MOVF      SLAVE,0
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVLW     MASTER
	MOVWF     CHP_CNT
	CALL      WRABC46
	INCFSZ    EE_ADR
	GOTO      C46P_LP
	GOTO	VERIC46

LOADC46
	CALL      CHK_LOCK
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0XC0
	MOVWF     EE_ADR
C46L_LP   MOVLW     MASTER    ;READ MASTER
	MOVWF     CHP_CNT
	CALL      RDABC46
	MOVF      SLAVE,0   ;WRITE SLAVE
	MOVWF     CHP_CNT
	CALL      WRABC66
	INCFSZ    EE_ADR    ;NEXT BYTE UNTIL 512 BYTE
	GOTO      C46L_LP
	GOTO      VERIC46

VERIC46
	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0XC0      ;COPY START FROM 0 BYTE
	MOVWF     EE_ADR
C46V_LP   MOVLW     MASTER    ;READ MASTER
	MOVWF     CHP_CNT
	CALL      RDABC46
	MOVF      DATA_H,0  ;SAVE IN DATA_VH
	MOVWF     DATA_VH
	MOVF      DATA_L,0  ;SAVE IN DATA_VL
	MOVWF     DATA_VL
	MOVF      SLAVE,0   ;READ SLAVE
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVF      DATA_VH,0
	SUBWF     DATA_H
	BTFSS     STATUS,2
	GOTO      ERROR
	MOVF      DATA_VL,0
	SUBWF     DATA_L
	BTFSS     STATUS,2
	GOTO      ERR46
	INCFSZ    EE_ADR    ;NEXT BYTE UNTIL 512 BYTE
	GOTO      C46V_LP
	MOVLW	B'01101110'         ;YES , OKAY
	MOVWF	BUFFER
	GOTO      DISC46
ERR46     MOVLW     B'01111001'	;E , ERROR
	MOVWF	BUFFER
DISC46	MOVLW   	MASTER
	MOVWF   	CHP_CNT
	CALL	EWDS
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWDS
	GOTO      WERM

WRABC46
	MOVF    	CHP_CNT,0
	MOVWF     LED             ;CHIP SELECT
	CALL      WRITE_CMD
	MOVF    	EE_ADR,0
	MOVWF   	TEMP    	;WRITE ADDRESS TO DI
	CALL	WR46ADR
	CALL	WR2BYT
	RETURN

RDABC46
	MOVF    	CHP_CNT,0	;ENABLE CHIP SELECTED
	MOVWF	LED
	CALL      READ_CMD
	MOVF    	EE_ADR,0  ;KEEPS ADDRESS
	MOVWF   	TEMP    	;SEND ADDRESS FIRST
	CALL	WR46ADR
	CALL	RD2BYT
	RETURN

WR46ADR
	MOVLW   	6
	MOVWF   	BYT_CNT
	RLF   	TEMP,1    ;SHIFT MSB TO CARRY
	RLF   	TEMP,1    ;SHIFT MSB TO CARRY
MOV_A1    BCF	EEROM,DI
	RLF   	TEMP,1    ;SHIFT MSB TO CARRY
	BTFSC     STATUS,0  ;TEST CARRY BIT, SET IF BIT = 1
	BSF	EEROM,DI
	CALL	CLOCK
	DECFSZ	BYT_CNT
	GOTO	MOV_A1
	BCF	EEROM,DI
	RETURN

PROGC01
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0X80
	MOVWF     EE_ADR
C01P_LP   MOVF      SLAVE,0
	MOVWF     CHP_CNT
	CALL      RDABC66
	CALL      WRABC01
	INCFSZ    EE_ADR
	GOTO      C01P_LP
	GOTO	VERIC01

LOADC01
	CALL      CHK_LOCK
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0XFF
	MOVWF     DATA_H
	MOVLW     0X80
	MOVWF     EE_ADR
C01L_LP   CALL      RDABC01
	MOVF      SLAVE,0   ;WRITE SLAVE
	MOVWF     CHP_CNT
	CALL      WRABC66
	INCFSZ    EE_ADR    ;NEXT BYTE UNTIL 512 BYTE
	GOTO      C01L_LP
	GOTO      VERIC01

VERIC01
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWEN
	MOVLW     0X80      ;COPY START FROM 80h BYTE
	MOVWF     EE_ADR
C01V_LP   CALL      RDABC01
	MOVF      DATA_L,0  ;SAVE IN DATA_VL
	MOVWF     DATA_VL
	MOVF      SLAVE,0   ;READ SLAVE
	MOVWF     CHP_CNT
	CALL      RDABC66
	MOVF      DATA_VL,0
	SUBWF     DATA_L
	BTFSS     STATUS,2
	GOTO      ERR01
	INCFSZ    EE_ADR
	GOTO      C01V_LP
	MOVLW	B'01101110'         ;YES , OKAY
	MOVWF	BUFFER
	GOTO      DISABL01
ERR01     MOVLW     B'01111001'	;E , ERROR
	MOVWF	BUFFER
DISABL01
	MOVF    	SLAVE,0
	MOVWF   	CHP_CNT
	CALL	EWDS
	GOTO      WERM

;************************************************************
RDABC01
	START_BIT
          SND_DRV_ADR
          CALL      CLOCK               ;dummy write
          WAIT_ACK
	SND_C01_ADR
          WAIT_ACK
          START_BIT
          SND_DRV_ADR

WRINS2    BSF       EEROM,SDA
	CALL      CLOCK
	SET_SDA_IN
	BSF       EEROM,SCL
	MOVLW     B'01111000'     ;NO, PRINT ERROR
	MOVWF	BUFFER          ;AND RESTART
	BTFSC     EEROM,SDA
	GOTO      WERM
	BCF       EEROM,SCL
	MOVLW   	8
	MOVWF   	BYT_CNT
RDC01LP   RLF	TEMP
	BSF       EEROM,SCL
	BCF   	TEMP,0
	NOP
	BTFSC	EEROM,SDA
	BSF	TEMP,0
	BCF       EEROM,SCL
	DECFSZ    BYT_CNT
	GOTO      RDC01LP
	MOVF      TEMP,0
	MOVWF     DATA_L
	SET_SDA_OUT
	BSF       EEROM,SDA
	STOP_BIT
	RETURN
;*********************************************************
WRABC01
	MOVF	DATA_L,0
	MOVWF	DATA_VL
	START_BIT
          SND_DRV_ADR
          CALL      CLOCK           ;WRITE WORD ADDRESS
          WAIT_ACK                  ;WAIT FOR ACKNOWLEDGE
	SND_C01_ADR
	BCF	EEROM,SDA	      ;WRITE INSTRUCTION
	CALL      CLOCK           ;WRITE SDA 0
	BSF	EEROM,SDA
	SET_SDA_IN
          WAIT_ACK
	BCF	EEROM,SDA
	MOVF	DATA_L,0
	MOVWF	TEMP
	SET_SDA_OUT
	MOVLW   	8
	MOVWF   	BYT_CNT
MOVC01    BCF	EEROM,SDA
	RLF   	TEMP,1          ;SHIFT MSB TO CARRY
	BTFSC     STATUS,0        ;TEST CARRY BIT, SET IF BIT = 1
	BSF	EEROM,DI
	CALL	CLOCK
	DECFSZ	BYT_CNT
	GOTO	MOVC01
	SET_SDA_IN
          ACK_IN
	SET_SDA_OUT
	STOP_BIT
DLY10MS
	movlw     08h
	movwf     0Eh
DLYL5     movlw     0FFh
	movwf     0Fh
DLYL6     decfsz    0FH
	goto      DLYL6
	decfsz    0Eh
	goto      DLYL5
	RETURN
;***********************************************************
DELAY
	MOVLW	02
	MOVWF	0DH
DLYL1	movlw     0FFh
	movwf     0Eh
DLYL2     movlw     0FFh
	movwf     0Fh
DLYL3     decfsz    0FH
	goto      DLYL3
	decfsz    0Eh
	goto      DLYL2
	decfsz    0Dh
	goto      DLYL1
	return
;***********************************************************
WRITE_CMD
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	BCF	EEROM,DI
	RETURN

READ_CMD
	CALL	CLOCK
	BSF	EEROM,DI
	CALL	CLOCK
	CALL	CLOCK
	BCF	EEROM,DI
	CALL	CLOCK
	RETURN

READ_EE
	MOVWF     EEADR
	BSF       STATUS,5
	BSF	EECON1,RD
	BCF	STATUS,5
	MOVF	EEDATA,0
	RETURN

WRITE_EE
	MOVWF	EEADR
	BSF	STATUS,5
	BSF	EECON1,WREN
	MOVLW	55H
	MOVWF	EECON2
	MOVLW	0AAH
	MOVWF	EECON2
	BSF	EECON1,WR
WAIT_EW2  BTFSS	EECON1,EEIF
	GOTO	WAIT_EW2
	BCF	EECON1,EEIF
	BCF	EECON1,WREN
	BCF	STATUS,5
	RETURN
          END